@@ -3,6 +3,7 @@
 """
 
 import datetime
+from typing import List, Union
 
 import dateutil.parser
 import dateutil.relativedelta as rdelta
@@ -13,10 +14,10 @@ from .__version__ import __author__, __copyright__, __email__, __license__, __ve
 
 class DateTimeRange:
     """
-    The class that represents the time range.
+    A class that represents a range of datetime.
 
-    :param datetime.datetime/str start: |param_start_datetime|
-    :param datetime.datetime/str end: |param_end_datetime|
+    :param datetime.datetime/str start_datetime: |param_start_datetime|
+    :param datetime.datetime/str end_datetime: |param_end_datetime|
 
     :Examples:
         :Sample Code:
@@ -31,12 +32,16 @@ class DateTimeRange:
                 2015-03-22T10:00:00+0900 - 2015-03-22T10:10:00+0900
 
     .. py:attribute:: start_time_format
+        :type: str
+        :value: "%Y-%m-%dT%H:%M:%S%z"
 
         Conversion format string for :py:attr:`.start_datetime`.
 
         .. seealso:: :py:meth:`.get_start_time_str`
 
     .. py:attribute:: end_time_format
+        :type: str
+        :value: "%Y-%m-%dT%H:%M:%S%z"
 
         Conversion format string for :py:attr:`.end_datetime`.
 
@@ -62,7 +67,7 @@ class DateTimeRange:
 
     def __repr__(self):
         if self.is_output_elapse:
-            suffix = " ({})".format(self.end_datetime - self.start_datetime)
+            suffix = f" ({self.end_datetime - self.start_datetime})"
         else:
             suffix = ""
 
@@ -399,7 +404,7 @@ class DateTimeRange:
                 600.0
         """
 
-        return self.__get_timedelta_sec(self.timedelta)
+        return self.timedelta.total_seconds()
 
     def set_start_datetime(self, value, timezone=None):
         """
@@ -424,16 +429,7 @@ class DateTimeRange:
                 2015-03-22T10:00:00+0900 - NaT
         """
 
-        if value is None:
-            self.__start_datetime = None
-            return
-
-        try:
-            self.__start_datetime = typepy.type.DateTime(
-                value, strict_level=typepy.StrictLevel.MIN, timezone=timezone
-            ).convert()
-        except typepy.TypeConversionError as e:
-            raise ValueError(e)
+        self.__start_datetime = self.__normalize_datetime_value(value, timezone)
 
     def set_end_datetime(self, value, timezone=None):
         """
@@ -457,16 +453,7 @@ class DateTimeRange:
                 NaT - 2015-03-22T10:10:00+0900
         """
 
-        if value is None:
-            self.__end_datetime = None
-            return
-
-        try:
-            self.__end_datetime = typepy.type.DateTime(
-                value, strict_level=typepy.StrictLevel.MIN, timezone=timezone
-            ).convert()
-        except typepy.TypeConversionError as e:
-            raise ValueError(e)
+        self.__end_datetime = self.__normalize_datetime_value(value, timezone)
 
     def set_time_range(self, start, end):
         """
@@ -583,10 +570,10 @@ class DateTimeRange:
 
         if not is_inversion:
             if self.__compare_timedelta(step, seconds=0) < 0:
-                raise ValueError("invalid step: expect greater than 0, actual={}".format(step))
+                raise ValueError(f"invalid step: expect greater than 0, actual={step}")
         else:
             if self.__compare_timedelta(step, seconds=0) > 0:
-                raise ValueError("invalid step: expect less than 0, actual={}".format(step))
+                raise ValueError(f"invalid step: expect less than 0, actual={step}")
 
         current_datetime = self.start_datetime
         while current_datetime <= self.end_datetime:
@@ -631,6 +618,89 @@ class DateTimeRange:
             end_time_format=self.end_time_format,
         )
 
+    def subtract(self, x):
+        """
+        Remove a time range from this one and return the result.
+
+        - The result will be ``[self.copy()]`` if the second range does not overlap the first
+        - The result will be ``[]`` if the second range wholly encompasses the first range
+        - The result will be ``[new_range]`` if the second range overlaps one end of the range
+        - The result will be ``[new_range1, new_range2]`` if the second range is
+          an internal sub range of the first
+
+        :param DateTimeRange x:
+            Range to remove from this one.
+        :return: List(DateTimeRange)
+            List of new ranges when the second range is removed from this one
+        :Sample Code:
+            .. code:: python
+
+                from datetimerange import DateTimeRange
+                dtr0 = DateTimeRange("2015-03-22T10:00:00+0900", "2015-03-22T10:10:00+0900")
+                dtr1 = DateTimeRange("2015-03-22T10:05:00+0900", "2015-03-22T10:15:00+0900")
+                dtr0.subtract(dtr1)
+        :Output:
+          .. parsed-literal::
+
+                [2015-03-22T10:00:00+0900 - 2015-03-22T10:05:00+0900]
+        """
+        overlap = self.intersection(x)
+        # No intersection, return a copy of the original
+        if not overlap.is_set() or overlap.get_timedelta_second() <= 0:
+            return [
+                DateTimeRange(
+                    start_datetime=self.start_datetime,
+                    end_datetime=self.end_datetime,
+                    start_time_format=self.start_time_format,
+                    end_time_format=self.end_time_format,
+                )
+            ]
+
+        # Case 2, full overlap, subtraction results in empty set
+        if (
+            overlap.start_datetime == self.start_datetime
+            and overlap.end_datetime == self.end_datetime
+        ):
+            return []
+
+        # Case 3, overlap on start
+        if overlap.start_datetime == self.start_datetime:
+            return [
+                DateTimeRange(
+                    start_datetime=overlap.end_datetime,
+                    end_datetime=self.end_datetime,
+                    start_time_format=self.start_time_format,
+                    end_time_format=self.end_time_format,
+                )
+            ]
+
+        # Case 4, overlap on end
+        if overlap.end_datetime == self.end_datetime:
+            return [
+                DateTimeRange(
+                    start_datetime=self.start_datetime,
+                    end_datetime=overlap.start_datetime,
+                    start_time_format=self.start_time_format,
+                    end_time_format=self.end_time_format,
+                )
+            ]
+
+        # Case 5, underlap, two new ranges are needed.
+        return [
+            DateTimeRange(
+                start_datetime=self.start_datetime,
+                end_datetime=overlap.start_datetime,
+                start_time_format=self.start_time_format,
+                end_time_format=self.end_time_format,
+            ),
+            DateTimeRange(
+                start_datetime=overlap.end_datetime,
+                end_datetime=self.end_datetime,
+                start_time_format=self.start_time_format,
+                end_time_format=self.end_time_format,
+            ),
+        ]
+
     def encompass(self, x):
         """
         Newly set a time range that encompasses
@@ -698,10 +768,65 @@ class DateTimeRange:
         self.__start_datetime += discard_time
         self.__end_datetime -= discard_time
 
-    def __validate_value(self, data_prop):
-        if data_prop.typecode not in [typepy.Typecode.DATETIME, typepy.Typecode.NONE]:
-            raise ValueError("invalid datetime value: {}".format(data_prop))
+    def split(self, separator: Union[str, datetime.datetime]) -> List["DateTimeRange"]:
+        """
+        Split the DateTimerange in two DateTimerange at a specifit datetime.
 
-    @staticmethod
-    def __get_timedelta_sec(dt):
-        return int(dt.days * 60 ** 2 * 24 + float(dt.seconds) + dt.microseconds / (1000.0 ** 2))
+        :param Union[str, datetime.datetime] separator:
+            Date and time to split the DateTimeRange.
+            This value will be included for both of the ranges after split.
+
+        :Sample Code:
+            .. code:: python
+
+                from datetimerange import DateTimeRange
+                dtr = DateTimeRange("2015-03-22T10:00:00+0900", "2015-03-22T10:10:00+0900")
+                dtr.split("2015-03-22T10:05:00+0900")
+        :Output:
+            .. parsed-literal::
+
+                [2015-03-22T10:00:00+0900 - 2015-03-22T10:05:00+0900,
+                2015-03-22T10:05:00+0900 - 2015-03-22T10:10:00+0900]
+        """
+
+        self.validate_time_inversion()
+
+        separatingseparation = self.__normalize_datetime_value(separator, timezone=None)
+
+        if (separatingseparation not in self) or (
+            separatingseparation in (self.start_datetime, self.end_datetime)
+        ):
+            return [
+                DateTimeRange(
+                    start_datetime=self.start_datetime,
+                    end_datetime=self.end_datetime,
+                    start_time_format=self.start_time_format,
+                    end_time_format=self.end_time_format,
+                )
+            ]
+
+        return [
+            DateTimeRange(
+                start_datetime=self.start_datetime,
+                end_datetime=separatingseparation,
+                start_time_format=self.start_time_format,
+                end_time_format=self.end_time_format,
+            ),
+            DateTimeRange(
+                start_datetime=separatingseparation,
+                end_datetime=self.end_datetime,
+                start_time_format=self.start_time_format,
+                end_time_format=self.end_time_format,
+            ),
+        ]
+
+    def __normalize_datetime_value(self, value, timezone):
+        if value is None:
+            return None
+
+        try:
+            return typepy.type.DateTime(
+                value, strict_level=typepy.StrictLevel.MIN, timezone=timezone
+            ).convert()
+        except typepy.TypeConversionError as e:
+            raise ValueError(e)
